unit SevenSegments;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, Graphics, math, BGRABitmap, BGRABitmapTypes;

procedure Draw7segs (C: TCanvas; s: string; xoff,yoff, FontHeight: integer; Color: TColor);

implementation


const Deg2Rad = pi/180;
      Height2Width = 0.7;

function shiftPoly (poly: ArrayOfTPointF; shiftX, shiftY : single) : ArrayOfTPointF;
var i : integer;
    begin
    setLength(result,length(poly));
    for i := 0 to high (poly) do
       begin
       if poly[i] = EmptyPointF then result[i] := EmptyPointF else
              result[i] := PointF(Poly[i].x+ShiftX,Poly[i].y + ShiftY);
       end; //
    end;


function rotatePoly(Poly: ArrayOfTPointF; angleDegrees: single; Center: TPointF) : ArrayOfTPointF;
var Sinus, Cosinus: single;
    P: TPointF;
    i: Integer;
begin
setLength(result,length(poly));
Math.SinCos(Deg2Rad*angleDegrees, Sinus, Cosinus);
for i := 0 to high(Poly) do
  begin
  if poly[i] = EmptyPointF then result[i] := EmptyPointF else
        begin
          P.x := Poly[i].x - Center.x;
          P.y := Poly[i].y - Center.y;
          Result[i].x := P.x *cosinus - P.y * sinus + Center.x;
          Result[i].y := P.x * sinus + P.y * cosinus + Center.y;
        end; // else
  end; // i
end;



// 7 Segment Digit Display
procedure drawOneDigit(dest: TBGRABitmap; ch: char; xoff,yoff, FontHi: single;
								        Color : TColor; out NextX : single);

// FontHi = 2*width+2*dist+height;
// dimension of one *vertical* segment
const  fheight = 0.2; // of width
       fXshift = 0.1; // of width
       fdist   = 0.08; // of width
       fWidthFactor= 2+2*fdist+fheight;  // = Fonthi : width

      SevenSegs : array[0..11,0..6] of byte =  (
           {    A  B  C  D  E  F  G }
           {0} (1, 1, 1, 1, 1, 1, 0),    //   7-segment-display
           {1} (0, 1, 1, 0, 0, 0, 0),    //     __A___
           {2} (1, 1, 0 ,1, 1, 0, 1),    //   F |    | B
           {3} (1, 1, 1, 1, 0, 0, 1),    //     |_G__|
           {4} (0, 1, 1, 0, 0, 1, 1),    //   E |    | C
           {5} (1, 0, 1, 1, 0, 1, 1),    //     |____|
           {6} (1, 0, 1, 1, 1, 1, 1),    //       D
           {7} (1, 1, 1 ,0, 0, 0 ,0),
           {8} (1, 1, 1, 1, 1, 1, 1),
           {9} (1, 1, 1, 1, 0, 1, 1),
           {-} (0, 0, 0, 0, 0, 0, 1),
           {E} (1, 0, 0, 1, 1, 1, 1)
              );

var
  Width, height, xshift, dist: single;
  poly, worker : ArrayOfTPointF;
  digit : byte;
  i: Integer;
  r : single;
begin
Width:= FontHi / fWidthFactor;
height := Width * fheight;
xshift := Width * fXshift;
dist   := Width * fdist;
if ch = '.' then
  begin
  r := FontHi / 20;
  dest.EllipseAntialias(xoff-3*r,yoff+FontHi-r,r,r,cssBlack,0.5,Color);
  nextX := XOff;
  Exit;
	end;

case ch of
'0'..'9' : digit := strToInt(ch);
'-'      : digit := 10;
'E'      : Digit := 11;
otherwise begin nextX := xoff; exit; end;
end; // case

(*  p5 _______________ p0
      /               \
 p4  /                 \ p1
     \                 /
   p3 \_______________/  p2
*)
setLength(Poly,6);
poly[0] := pointF (Width-xshift, 0);
poly[1] := pointF (Width, height/2);
poly[2] := pointF (Width-xshift, height);
poly[3] := pointF (xshift, height);
poly[4] := pointF (0, height/2);
poly[5] := pointF (xshift, 0);
// put each of the 7 segments into position
for i := 0 to  6 do
   begin
     case i of
{A}  0: worker := shiftpoly (poly,xoff,yoff);
{B}  1: begin worker := rotatePoly(poly,90,poly[4]);
              worker := shiftPoly(worker, width+xoff,yoff+dist); end;
{C}  2: begin worker :=rotatePoly(poly,90,poly[4]);
              worker:=shiftPoly(worker, width+xoff,yoff+2*dist+width); end;
{D}  3: worker := shiftpoly (poly,xoff,yoff+2*width+2.5*dist);
{E}  4: begin worker := rotatePoly(poly,90,poly[4]);
              worker := shiftPoly(worker, xoff,yoff+2*dist+width); end;
{F}  5: begin worker := rotatePoly(poly,90,poly[4]); worker := shiftPoly(worker, xoff,yoff+dist); end;
{G}  6: worker := shiftpoly (poly,xoff,yoff+width+1.5*dist);
     end; // case
     if SevenSegs[digit,i] = 1 then
            dest.DrawPolygonAntialias(worker,cssBlack,0.5,Color) else
            dest.DrawPolygonAntialias(worker,BGRA(199,199,199),0.5,BGRA(240,240,240));
   end;// i
nextX := xoff + FontHi * Height2Width;
end;
// =================

procedure Draw7segs (C: TCanvas; s: string; xoff,yoff, FontHeight: integer; Color: TColor);
var bmp: TBGRABitmap;
    nextX : single;
    DotSpace,wi,hi,i: integer;
    {$Ifdef Windows}
    RSource, RDest: TRect ;
    {$endif}
begin
DotSpace := trunc(4*FontHeight / 20)+1 ;
wi := length(s)*FontHeight+DotSpace;// in case of leading dot
Hi := round(wi*Height2Width);
bmp := TBGRABitmap.create (wi,hi);
nextX := Dotspace;

for i := 1 to length(s) do
    begin
    drawOneDigit(bmp,s[i],nextX,0,FontHeight,color, nextX);
    end;
  {$Ifdef Windows }
  Rsource := Rect(0,0,bmp.width,bmp.height);
  Rdest := Rect (xoff,yoff,xoff+bmp.width,yoff+bmp.height);
  C.copyRect(RDest,bmp.canvas,RSource);
  {$else}
  bmp.draw(C,xoff,yoff,false);
  {$endif}
  bmp.free;
end;



end.

